//same colors as on previous charts
window.chartColors = ["#edc240", "#afd8f8", "#cb4b4b", "#4da74d", "#9440ed"];
//for dashboard bars
window.lightGreen = "#95bb31";
window.TimeConstants = {
  DAY_OF_MONTH: {javaInterval: 5, momentIncrement: 'days', chartsMinUnit: 'day'},
  MONTH: {javaInterval: 2, momentIncrement: 'months', chartsMinUnit: 'month'},
  YEAR: {javaInterval: 1, momentIncrement: 'years', chartsMinUnit: 'year'},
  findByInterval: function (interval) {
    var foundValue = undefined;
    $.each(this, function(key, value) {
      if (value.javaInterval === interval) {
        foundValue = value;
      }
    });
    return foundValue;
  }
};

(function() {
  if (LS.chart) {
    return;
  }
  var $conf = $('#boxWithData');
  LS.chart = {
    type: $conf.data('chartType'),
    useBars: $conf.data('chartBars') === 'true',
    totalTokenId: parseInt($conf.data('chartTotalToken'), 10)
  };
})();

(function() {
  moment.locale('en');
  var chart = {};

  LS.chart.createChart = function(type, data){
    chart = {
      plot: null,
      seriesOpts: {},
      from: LS.chart.from,
      to: LS.chart.to,
      interval: LS.chart.interval,
      type: type,
      seriesDataSets: data,

      /**
       * Initialization method. Called on chart creation
       */
      init: function() {
        var that = this;
        this.seriesOpts = {
          options: {
            legend: {
              display: false
            },
            tooltips: {
              enabled: true,
              displayColors: false,
              callbacks: {
                //show all data in tooltip header
                title: function (tooltipItem) {
                  var item = tooltipItem[0];
                  return moment(item.xLabel).format('MMM D') + " - " + that.createValueLabel(item.yLabel);
                },
                //Disable default way to show tooltip
                label: function () {
                  return null;
                }
              }
            },
            scales: {
              xAxes: [{
                type: 'time',
                time: {
                  minUnit: 'day',
                  round: 'day'
                },
                gridLines: {
                  //make zeroline be of the same color as others, otherwise first grid line is bolder
                  zeroLineColor: 'rgba(0, 0, 0, 0.1)'
                }
              }],
              yAxes: [{
                ticks: {
                  beginAtZero:true,
                  min: 0,
                  callback: function(value, index, values) {
                    //filter out non integer numbers
                    if (Math.floor(value) === value) {
                      return that.yTickLabel(value, index, values);
                    }
                  }
                }
              }]
            }
          },
          data: {
            datasets: []
          },
          type: 'line'
        };
        this.timesavedChart = that.type.toLowerCase().indexOf('timesaved') !== -1;
        if(this.timesavedChart) {
          this.convertValue = function (value) {
            return moment.duration(value).asHours();
          };
          this.yTickLabel = function(value) {
            return value + "h";
          };
          this.createValueLabel = function (hours) {
            var flooredHours = Math.floor(hours);
            if (flooredHours === 0) {
              return Math.floor(hours * 60) + " minutes saved";
            }
            else if (flooredHours === 1) {
              return flooredHours + " hour saved";
            }
            return flooredHours + " hours saved";
          };
        }
        //TODO refactor creation to separate methods to be less messy.
        //TODO Basically there will be 2 types of charts bars timesaved and line seats in use.
        if (LS.chart.useBars) {
          this.seriesOpts.type = 'bar';
          this.seriesOpts.options.scales.xAxes = [{type: 'category'}];
          this.seriesOpts.options.tooltips.callbacks.title = function (tooltipItem) {
                var item = tooltipItem[0];
                return item.xLabel + " - " + that.createValueLabel(item.yLabel);
              };
          //bar chart is only on dashboard now and has no interaction, so this is valid.
          var tempDataset = this.seriesDataSets.map(function (line) {
            return that.normalizeToChartDataset(line);
          });
          that.seriesOpts.options.scales.xAxes[0].gridLines = {
            display: false
          };
          that.seriesOpts.data.labels = tempDataset[0].data.map(function (point) {
            return moment(point.x).format('MMM D');
          });
          that.seriesOpts.data.datasets = [
            {
              label: tempDataset.label,
              borderColor: window.lightGreen,
              backgroundColor: window.lightGreen,
              data: tempDataset[0].data.map(function (point) {
                return point.y;
              })
            }
          ];
          if (LS.chart.dashboard) {
            //disabling assign colors for dashboard bar chart
            this.assignColors = function() {};
          }
        } else {
          that.seriesOpts.data.datasets = this.seriesDataSets.map(function (line) {
            return that.normalizeToChartDataset(line);
          });
          //workaround to set max yAxis a bit above max value (problem for small numbers, line goes directly on up bound)
          var values = that.seriesOpts.data.datasets.map(function (dataset) {
            return dataset.data
              .map(function (point) { return point.y;})
              .reduce(function(a,b) { return Math.max(a,b);});
          });
          var maxValue = values.reduce(function(a,b) { return Math.max(a,b);});
          this.seriesOpts.options.scales.yAxes[0].ticks.suggestedMax = Math.max(maxValue * 1.05, 1);
        }

        // associate it with either 'chart-timeSaved' or 'chart-seatsInUse' element.
        var ctx = document.getElementById("chart-" + type);
        this.assignColors();
        this.plot = new Chart(ctx, this.seriesOpts);
      },
      convertValue: function (value) {
        return value;
      },
      createValueLabel: function (value) {
        return value;
      },
      yTickLabel: function(value) {
        return value;
      },
      normalizeToChartDataset: function(seriesData) {
        var that = this;
        var values = seriesData.data.map(function (point) {
          return {x: moment(point[0]), y:that.convertValue(point[1])};
        });

        if (values.length === 1) {
          //if we have only one data point it will be shown on the left border by default which does not look nice.
          //we want it to appear in the middle, so we add two invisible fake points (before and after)
          var postTick = this.getNextTick(moment(values[0].x), 1);
          var prevTick = this.getNextTick(moment(values[0].x), -1);
          values = [{x: prevTick, y:null}, values[0], {x: postTick, y:null}];
        }

        return {
          label: seriesData.label,
          data: values,
          fill: false
        };
      },
      getNextTick: function(date, increment) {
        if (!increment) {
          increment = 1;
        }
        var that = this;
        var constant = TimeConstants.findByInterval(that.interval);
        date.add(increment, constant.momentIncrement);
        return date;
      },
      fullUrl: function(idsList){
        var that = this;
        var param = $.param({
          ids: idsList.join(","),
          interval: that.interval,
          from: that.from.format('YYYY-MM-DD'),
          to: that.to.format('YYYY-MM-DD')
        });
        return LS.routes[that.type + 'DataUrl'] + '?' + param;
      },
      fullCsvUrl: function() {
        var that = this;
        return LS.routes[that.type + 'CsvUrl'] + '?' + $.param({
          ids: _.map(chart.plot.data.datasets, function(serie){
            return serie.label;
          }).join(","),
          interval: that.interval,
          from: that.from.format('YYYY-MM-DD'),
          to: that.to.format('YYYY-MM-DD')
        });
      },
      // Reset the charts object (draw it again)
      reset: function() {
        var that = this;
        var ids = _.map(chart.plot.data.datasets, function(serie){
          return serie.label;
        });
        $.getJSON(this.fullUrl(ids), function (response) {
          var constant = TimeConstants.findByInterval(that.interval);
          if (!constant) {
            that.seriesOpts.options.scales.xAxes[0].time.minUnit = 'minute';
          } else {
            that.seriesOpts.options.scales.xAxes[0].time.minUnit = constant.chartsMinUnit;
          }
          that.seriesOpts.data.datasets = response.map(function (line) {
            return that.normalizeToChartDataset(line);
          });
          that.assignColors();
          that.plot.update();
        });
      },
      assignColors: function() {
        var series = this.seriesOpts.data.datasets;
        $("#legend-" + this.type + " .ls-chart-legend-item .ls-chart-legend-color").css("background", "lightgrey");
        for (var i = 0; i < series.length; ++i){
          var legendItem = $("#legend-" + this.type + "-item-" + series[i].label);
          var colorId = legendItem.data("color");
          var color = chartColors[colorId % chartColors.length];
          series[i].borderColor = color;
          series[i].backgroundColor = color;
          legendItem.find(".ls-chart-legend-color").css("background", color);
        }
      },
      downloadCsv: function(){
        window.open(this.fullCsvUrl());
      },
      toggleTokenChart: function(id){
        var data = this.seriesOpts.data.datasets;
        var that = this;
        for (var i = 0; i < data.length; i++) {
          if (data[i].label === id) {
            data.splice(i, 1);
            that.assignColors();
            that.plot.update();
            return;
          }
        }

        $.getJSON(chart.fullUrl([id]), function (response) {
          var data = that.seriesOpts.data.datasets;
          data.push(chart.normalizeToChartDataset(response[0]));
          that.assignColors();
          that.plot.update();
        });
      }
    };
    return chart;
  };

  var setUpControls = function(type) {
    var legendItem = $("#legend-" + type + " .ls-chart-legend-item");
    legendItem.addClass("ls-chart-legend-control");
    legendItem.click(function () {
      var id = $(this).data("id");
      chart.toggleTokenChart(id);
    });

    $("#interval-" + type).change(function () {
      chart.interval = parseInt($(this).val());
      chart.reset();
    });

    var calendarEnterListenerAdded = false;

    $('#date-range-field-' + type).daterangepicker({
      startDate: LS.chart.from,
      endDate: LS.chart.to,
      opens: "left",
      maxDate: LS.chart.to,
      autoUpdateInput: true,
      autoApply: true,
      locale: {
        format: 'DD MMMM, YYYY',
        firstDay: 1
      }
    }, function(start, end) {
      chart.from = start;
      chart.to = end;
      chart.reset();
    })
        .on('showCalendar.daterangepicker', function(ev, picker) {
          if (calendarEnterListenerAdded) {
            return;
          }
          calendarEnterListenerAdded = true;

          picker.container.find('input').on('keydown.daterangepicker', function(e){
            // enter pressed
            if (e.keyCode === 13) {
              picker.formInputsChanged(e);
              picker.clickApply(e);
            }
          });
        });

    $("#downloadCsv-" + type).click(function(){
      chart.downloadCsv();
    });
  };

  if (!LS.chart.dashboard) {
    //for Dashboard it is created from dashboard.js
    LS.chart.to = moment();
    LS.chart.from = moment(LS.chart.to).subtract(14, 'days');
    LS.chart.interval = TimeConstants.DAY_OF_MONTH.javaInterval;

    if (!(LS.chart.disabledControls && LS.chart.disabledControls[LS.chart.type])) {
      setUpControls(LS.chart.type);
    }
    var url = LS.routes[LS.chart.type + 'DataUrl'] + '?' + $.param({
      ids: LS.chart.totalTokenId,
      interval: LS.chart.interval,
      from: LS.chart.from.format('YYYY-MM-DD'),
      to: LS.chart.to.format('YYYY-MM-DD')
    });
    $.getJSON(url).success(function (data) {
      LS.chart.createChart(LS.chart.type, data).init();
    })
    .complete(function(){
      $('#boxWithData').find('i.fa-spinner').remove();
    });
  }
})();



